﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Globalization;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using Infragistics.Excel;
using Infragistics.Windows.DataPresenter.ExcelExporter;
using Microsoft.Win32;

namespace WpfExport2Excel
{
  /// <summary>
  /// Interaction logic for Window1.xaml
  /// </summary>
  public partial class Window1 : Window
  {
    private ObservableCollection<MyDataObject> _dataObjects;
    private DataPresenterExcelExporter _excelExporter;

    public Window1()
    {
      InitializeComponent();
      _excelExporter = new DataPresenterExcelExporter();
      _excelExporter.BeginExport += new EventHandler<BeginExportEventArgs>(_excelExporter_BeginExport);
      _excelExporter.EndExport += new EventHandler<EndExportEventArgs>(_excelExporter_EndExport);
      _excelExporter.CellExporting += new EventHandler<CellExportingEventArgs>(_excelExporter_CellExporting);
      _excelExporter.HeaderAreaExporting += new EventHandler<HeaderAreaExportingEventArgs>(_excelExporter_HeaderAreaExporting);
      _excelExporter.HeaderLabelExporting += new EventHandler<HeaderLabelExportingEventArgs>(_excelExporter_HeaderLabelExporting);
      _excelExporter.RecordExporting += new EventHandler<RecordExportingEventArgs>(_excelExporter_RecordExporting);
      _excelExporter.InitializeRecord += new EventHandler<InitializeRecordEventArgs>(_excelExporter_InitializeRecord);
      myGrid.SelectedItemsChanged += new EventHandler<Infragistics.Windows.DataPresenter.Events.SelectedItemsChangedEventArgs>(myGrid_SelectedItemsChanged);
      Loaded += Window1_Loaded;
    }

    void myGrid_SelectedItemsChanged(object sender, Infragistics.Windows.DataPresenter.Events.SelectedItemsChangedEventArgs e)
    {
    }

    void _excelExporter_InitializeRecord(object sender, InitializeRecordEventArgs e)
    {
      if (!e.Record.IsSelected)
      {
        System.Diagnostics.Trace.Assert(!myGrid.SelectedItems.Records.Contains(e.Record));
        e.SkipRecord = true;
        e.SkipDescendants = true;
      }
    }

    void _excelExporter_RecordExporting(object sender, RecordExportingEventArgs e)
    {
    }

    void _excelExporter_HeaderLabelExporting(object sender, HeaderLabelExportingEventArgs e)
    {
      if (String.Equals(e.Label, "Created"))
      {
        e.Cancel = true;
        return;
      }
      System.Diagnostics.Debugger.Break();
    }

    void _excelExporter_HeaderAreaExporting(object sender, HeaderAreaExportingEventArgs e)
    {
    }

    void _excelExporter_CellExporting(object sender, CellExportingEventArgs e)
    {
      if (String.Equals(e.Field.Name, "Created"))
      {
        e.Cancel = true;
        return;
      }
    }

    void _excelExporter_EndExport(object sender, EndExportEventArgs e)
    {
    }

    void _excelExporter_BeginExport(object sender, BeginExportEventArgs e)
    {
    }


    private void Window1_Loaded(object sender, RoutedEventArgs e)
    {
      _dataObjects = new ObservableCollection<MyDataObject>();
      _dataObjects.Add(new MyDataObject("First", DateTime.Now, DateTime.UtcNow));

      var date = DateTime.Now.AddHours(-9);
      _dataObjects.Add(new MyDataObject("Second", date, date.ToUniversalTime()));

      date = DateTime.Now.AddMinutes(-67);
      _dataObjects.Add(new MyDataObject("Third", date, date.ToUniversalTime()));

      date = new DateTime(2009, 4, 2, 17, 22, 33, 45);
      _dataObjects.Add(new MyDataObject("fourth", date, date.ToUniversalTime()));

      date = new DateTime(2009, 4, 2, 17, 22, 33, 55);
      _dataObjects.Add(new MyDataObject("fifth", date, date.ToUniversalTime()));

      myGrid.DataSource = _dataObjects;
    }

    private void Button_Click(object sender, RoutedEventArgs e)
    {
      var saveDialog = new SaveFileDialog
      {
        AddExtension = true,
        DefaultExt = "xlsx",
        Filter = "Excel Worksheets (*.xlsx)|*.xlsx|All Files|*.*"
      };
      if (saveDialog.ShowDialog() == true)
      {
        //var excelExporter = new DataPresenterExcelExporter();
        _excelExporter.Export(myGrid, saveDialog.FileName, WorkbookFormat.Excel2007);
        //Workbook workBook = _excelExporter.Export(myGrid, new Workbook(WorkbookFormat.Excel2007), 10, 1);
        //workBook.Save(saveDialog.FileName);
      }

    }
  }

  public class MyDataObject
  {
    public string Name { get; set; }
    public DateTime Created { get; set; }
    public DateTime Modified { get; set; }

    public MyDataObject(string name, DateTime created)
    {
      Name = name;
      Created = created;
    }

    public MyDataObject(string name, DateTime created, DateTime modified)
      : this(name, created)
    {
      Modified = modified;
    }
  }

  public class UTCToLocalTimeConverter : IValueConverter
  {
    #region IValueConverter Members

    public object Convert(object value, Type targetType, object parameter, CultureInfo culture)
    {
      if (value == null) return value;

      var d = (DateTime)value;
      return d.ToLocalTime();
    }

    public object ConvertBack(object value, Type targetType, object parameter, CultureInfo culture)
    {
      if (value == null) return value;

      var d = (DateTime)value;
      return d.ToUniversalTime();
    }

    #endregion
  }

}
